/*********************************************************************************
 *
 * File name:		main.cpp
 * Version:			2.0
 *
 * Software:		Power Supply On-Chip
 * Authors:			Xuan Wang, Jiang Xu, Zhe Wang, Haoran Li, Rafael K.V. Maeda, Xiaowen Wu, Zhehui Wang, Peng Yang and Luan H.K. Duong
 *					
 * Website:			http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include <string.h>
#include <assert.h>
#include <iostream>
#include <math.h>
#include <stdlib.h>
#include <sstream>
#include <vector>
#include <stdio.h>
#include <string>


using namespace std;


int main(int argc, char* argv[]) {

int num_param_type_1 = 0;
int num_param_type_2 = 0;

int num_variable_1 = 12;
int num_variable_2 = 13;

vector < float > param_type_1;
vector < float > param_type_2;

vector < int > allocate_VR;
vector < float > inter_level;

string end_str = "*/";
char temp_str[500];


// the temperature coefficient of resistance
vector < float > material_TC;
// Aluminium
material_TC.push_back(0.0039);
// Copper
material_TC.push_back(0.0039);
// Gold
material_TC.push_back(0.0034);


// read the configuration file


FILE *fp=fopen("./cfg_proc.txt", "r");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

vector < float > voltage;
vector < float > current;
vector < vector < vector < int > > > alloc_VR;
vector < int > num_VR;
vector < float > con_area;

int num_stage = 1;
int num_on_chip_domain = 1;
int num_on_chip_VR = 1;
int num_off_chip_VR = 1;
int temp_val1 = 1;
int temp_val2 = 1;
int temp_val3 = 1;
float temp_val = 0.0;
float power_proc = 0.0;
float temp_vout = 0.0;


// ignore the copyright information
	
while(1){
	fgets(temp_str, 500, fp);
	string tstr(temp_str); 
	if(tstr.find(end_str)!=string::npos){
		break;
	}
}


// read the number of the on-chip power domains
fscanf(fp, "%d", &num_on_chip_domain);


// read the voltage levels and the average current of each power domain 
for (int i=0; i<num_on_chip_domain; i++) {

	fscanf(fp, "%f", &temp_val);
	voltage.push_back(temp_val);

	temp_vout = temp_val;

	fscanf(fp, "%f", &temp_val);
	current.push_back(temp_val);

	power_proc = power_proc + temp_vout*temp_val;
}


// get the settings of the intermediate voltage levels for the decomposition method


fscanf(fp, "%d", &temp_val1);

for (int i=0; i<temp_val1; i++) {

	fscanf(fp, "%d", &temp_val2);
	inter_level.push_back(temp_val2);
	fscanf(fp, "%f", &temp_val);
	inter_level.push_back(temp_val);
	fscanf(fp, "%f", &temp_val);
	inter_level.push_back(temp_val);
}


// get the supply voltage of the energy source of the power delivery system

float Vs = 12.0;

fscanf(fp, "%f", &temp_val);

Vs = temp_val;

fclose(fp);

// we will facilitate load balance in the later


fp=fopen("./cfg_PDS.txt", "r");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

// ignore the copyright information
	
while(1){
	fgets(temp_str, 500, fp);
	string tstr(temp_str); 
	if(tstr.find(end_str)!=string::npos){
		break;
	}
}


// read the number of the stage 
fscanf(fp, "%d", &num_stage);

for (int k=0; k<num_stage; k++) {

	// read the number of the on-chip voltage regulators
	
	fscanf(fp, "%d", &temp_val1);
	num_VR.push_back(temp_val1);

	// read the allocation of on-chip power domains to different on-chip VRs

	vector < vector < int > > temp_vector_1;

	for (int i=0; i<num_VR.at(k); i++) {
	
		fscanf(fp, "%d", &temp_val1);

		vector < int > temp_vector_2;

		temp_vector_2.resize(temp_val1,0);

		for (int j=0; j<temp_val1; j++) {
	
			fscanf(fp, "%d", &temp_val2);
	
			temp_vector_2.at(j) = temp_val2;
		}

		temp_vector_1.push_back(temp_vector_2);

		allocate_VR.push_back(0);

	}

	alloc_VR.push_back(temp_vector_1);

	fscanf(fp, "%f", &temp_val);
	con_area.push_back(temp_val);

}


// check whether kth stage VRs contained all the VRs at (k-1)th stage
temp_val1 = 0;

for (int i=0; i<num_VR.at(0); i++) {

	temp_val1 += alloc_VR.at(0).at(i).size();

}

assert(temp_val1 == num_on_chip_domain);


for (int k=1; k<num_stage; k++) {

	temp_val1 = 0;

	for (int i=0; i<num_VR.at(k); i++) {

		temp_val1 += alloc_VR.at(k).at(i).size();

	}

	assert(temp_val1 == num_VR.at(k-1));

}

fclose(fp);

// read the design constraints

fp=fopen("./cfg_con.txt", "r");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

vector < float > con_VR;
vector < float > con_domain;
vector < float > temp_VR;

// ignore the copyright information
	
while(1){
	fgets(temp_str, 500, fp);
	string tstr(temp_str); 
	if(tstr.find(end_str)!=string::npos){
		break;
	}
}


for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
		// read the channel width constraint first
		// the lower bound first, and then the upper bound
		// the channel width, frequency, the inductor and capacitor
	
		for (int k=0; k<4; k++) {
			fscanf(fp, "%f", &temp_val);
			con_VR.push_back(temp_val);
			fscanf(fp, "%f", &temp_val);
			con_VR.push_back(temp_val);
		}

		// the minimum output ripple percent and transient voltage drop percent
		fscanf(fp, "%f", &temp_val);
		con_VR.push_back(temp_val);
		fscanf(fp, "%f", &temp_val);
		con_VR.push_back(temp_val);

		// the temperature of the regulator
		fscanf(fp, "%f", &temp_val);
		temp_VR.push_back(temp_val);

	}
}

fclose(fp);

// read technology files of the regulator design 


fp=fopen("./param_VR.txt", "r");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

// ignore the copyright information
	
while(1){
	fgets(temp_str, 500, fp);
	string tstr(temp_str); 
	if(tstr.find(end_str)!=string::npos){
		break;
	}
}


// read the cfg about number of types of regulator used, and the name of their separate
// configuration files


int num_cfg_VR = 1;
char cfg_VR_name[100];
//char temp_str [100];


int cfg_VR = 0; 

fscanf(fp, "%d", &num_cfg_VR);


for (int i=0; i<num_cfg_VR; i++) {
	
	fscanf(fp, "%s", &cfg_VR_name);

	// load the parameters of different kinds of VRs
	
	FILE *fp2=fopen(cfg_VR_name, "r");


	if (fp2 == NULL) {
		cout << "Error opening file" << endl;
		return 0;
	}

	// ignore the copyright information
	
	while(1){
		fgets(temp_str, 500, fp2);
		string tstr(temp_str); 
		if(tstr.find(end_str)!=string::npos){
			break;
		}
	}
	
	fscanf(fp2, "%d", &temp_val1);

	if (temp_val1 == 1) {
	
		fscanf(fp2, "%d", &num_param_type_1);

		// record the parameters of bulky buck converter in a predefined order
		for (int i=0; i<num_param_type_1; i++) {
	
			fscanf(fp2, "%f", &temp_val);

			param_type_1.push_back(temp_val);
		
		}
		
		// record the allocation of the VRs using this type
		fscanf(fp2, "%d", &temp_val3);

		for (int i=0; i<temp_val3; i++) {
	
			fscanf(fp2, "%d", &temp_val2);

			allocate_VR.at(temp_val2)=temp_val1;
		
		}

	}
	else if (temp_val1 == 2) {
	
		fscanf(fp2, "%d", &num_param_type_2);

		// record the parameters of integrated buck converter in a predefined order
		for (int i=0; i<num_param_type_2; i++) {
	
			fscanf(fp2, "%f", &temp_val);

			param_type_2.push_back(temp_val);
		
		}
		
		// record the allocation of the VRs using this type
		fscanf(fp2, "%d", &temp_val3);

		for (int i=0; i<temp_val3; i++) {
	
			fscanf(fp2, "%d", &temp_val2);

			allocate_VR.at(temp_val2)=temp_val1;
		
		}
	}
	else {
		
		assert (1 != 1);

	}

	fclose(fp2);
}

fclose(fp);

// read the total parasitic impedance between different stage of VRs 

vector < float > param_PDN;
vector < float > material_PDN;
vector < float > temp_PDN;

fp=fopen("./param_PDN.txt", "r");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

// ignore the copyright information
	
while(1){
	fgets(temp_str, 500, fp);
	string tstr(temp_str); 
	if(tstr.find(end_str)!=string::npos){
		break;
	}
}


// read the parasitic impedance at the output of each VR

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		fscanf(fp, "%f", &temp_val);
		param_PDN.push_back(temp_val);

		fscanf(fp, "%f", &temp_val);
		material_PDN.push_back(temp_val);

		fscanf(fp, "%f", &temp_val);
		temp_PDN.push_back(temp_val);

	}

}

for (int i=0; i<num_VR.at(num_stage-1); i++) {

	fscanf(fp, "%f", &temp_val);
	param_PDN.push_back(temp_val);

	fscanf(fp, "%f", &temp_val);
	material_PDN.push_back(temp_val);

	fscanf(fp, "%f", &temp_val);
	temp_PDN.push_back(temp_val);

}

fclose(fp);


// generate the matlab file for convex optimization by writing a convex matlab script into the output file.
fp=fopen("./PDS_optimization.m", "w");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

fprintf(fp, "clear\n");

fprintf(fp, "minimum = 1E6;\n");
fprintf(fp, "power_proc = %E;\n", power_proc);
fprintf(fp, "Vs = %E;\n", Vs);

int index_VR = 0;

for (int i=0; i<num_stage; i++) {

	// describe the 1st stage VRs and on-chip power domains
	if (i == 0) {

		for (int j=0; j<num_VR.at(i); j++) {
			
			fprintf(fp, "Vout_%d = max([ ", index_VR);

			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
			
				fprintf(fp, "%E, ", voltage.at(alloc_VR.at(i).at(j).at(k)));
			
			}

			fprintf(fp, "0.0 ]);\n");

			fprintf(fp, "Iout_%d = ", index_VR);

			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
			
				fprintf(fp, "%E + ", current.at(alloc_VR.at(i).at(j).at(k)));
			
			}

			fprintf(fp, "0 ;\n");

			index_VR ++;

		}

		// print the for statement for evaluating different combinations of intermediate voltage levels
		fprintf(fp, "intermediate_levels = [ ");
		for (int i=0; i<(inter_level.size()/3); i++) {
			fprintf(fp, "%d ", inter_level.at(3*i));
			fprintf(fp, "%E ", inter_level.at(3*i+1));
			fprintf(fp, "%E ", inter_level.at(3*i+2));
		}
		fprintf(fp, "];\n");

		for (int i=0; i<(inter_level.size()/3); i++) {
			
			fprintf(fp, "for i_%d = 1:1:%f\n", i, inter_level.at(3*i));

		}

		if (num_stage > 1) {

			for (int i=0; i<(inter_level.size()/3); i++) {
			
				if (inter_level.at(3*i) == 1) {

					fprintf(fp, "Vout_%d = %E;\n", index_VR+i, inter_level.at(3*i+1));

				}
				else {

					fprintf(fp, "Vout_%d = %E + (i_%d-1)*(%E-%E)/(%f-1);\n", index_VR+i, inter_level.at(3*i+1), i, inter_level.at(3*i+2), inter_level.at(3*i+1), inter_level.at(3*i));

				}

			}
		}
		else {
		
			for (int i=0; i<(inter_level.size()/3); i++) {
			
				if (inter_level.at(3*i) == 1) {

					fprintf(fp, "Vin_%d = %E;\n", i, inter_level.at(3*i+1));

				}
				else {

					fprintf(fp, "Vin_%d = %E + (i_%d-1)*(%E-%E)/(%f-1);\n", index_VR+i, inter_level.at(3*i+1), i, inter_level.at(3*i+2), inter_level.at(3*i+1), inter_level.at(3*i));

				}

			}
		
		}

	}
	else if ((i>0) && (i<num_stage)) {
	
		for (int j=0; j<num_VR.at(i); j++) {
			
			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
			
				fprintf(fp, "Vin_%d = Vout_%d;\n", alloc_VR.at(i).at(j).at(k), index_VR);
			
			}

			if (i == num_stage -1) {
			
				fprintf(fp, "Vin_%d = %E;\n", index_VR, Vs);
			
			}

			index_VR ++;
		
		}
	
	}
}


fprintf(fp, "cvx_begin gp\n");

fprintf(fp, "variables ");

// write the variables of VRs in the used in the optimization

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
		
		if (i == 0) {
			
			if ((num_stage==1) && (j == num_VR.at(0)-1)) {
			
				fprintf(fp, "W_%d Ir_%d f_%d Rind_%d Rds_%d L_%d D_%d C_%d K_%d Iin_%d A_%d delta_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);

			}
			else {
			
				fprintf(fp, "W_%d Ir_%d f_%d Rind_%d Rds_%d L_%d D_%d C_%d K_%d Iin_%d A_%d delta_%d ", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);

			}
		
		}
		else if ((i == num_stage -1) && (j == num_VR.at(i)-1)) {
		
			fprintf(fp, "W_%d Ir_%d f_%d Rind_%d Rds_%d L_%d D_%d C_%d K_%d Iin_%d Iout_%d A_%d delta_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
		}
		else {
		
			fprintf(fp, "W_%d Ir_%d f_%d Rind_%d Rds_%d L_%d D_%d C_%d K_%d Iin_%d Iout_%d A_%d delta_%d ", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
		}

		index_VR ++;

	}
}

fprintf(fp, "minimize ");

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		if ((i == num_stage -1) && (j<num_VR.at(i)-1)) {
		
			fprintf(fp, "Iin_%d + ", index_VR);
		
		}
		else if ((i == num_stage -1) && (j == num_VR.at(i)-1)) {
		
			fprintf(fp, "Iin_%d;\n", index_VR);
		
		}
		else {
		}

		index_VR ++;
	}	

}

fprintf(fp, "subject to\n");

fprintf(fp, "cvx_precision high\n");

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		// print the design constraint of the VRs
		fprintf(fp, "D_%d <= 1;\n", index_VR);
		if (con_VR.at(10*index_VR+0) != -1) {
		
			fprintf(fp, "W_%d >= %E;\n", index_VR, con_VR.at(10*index_VR+0));

		}
		if (con_VR.at(10*index_VR+1) != -1) {
		
			fprintf(fp, "W_%d <= %E;\n", index_VR, con_VR.at(10*index_VR+1));

		}
		if (con_VR.at(10*index_VR+2) != -1) {
		
			fprintf(fp, "f_%d >= %E;\n", index_VR, con_VR.at(10*index_VR+2));

		}
		if (con_VR.at(10*index_VR+3) != -1) {
		
			fprintf(fp, "f_%d <= %E;\n", index_VR, con_VR.at(10*index_VR+3));

		}
		if (con_VR.at(10*index_VR+4) != -1) {
		
			fprintf(fp, "L_%d >= %E;\n", index_VR, con_VR.at(10*index_VR+4));

		}
		if (con_VR.at(10*index_VR+5) != -1) {
		
			fprintf(fp, "L_%d <= %E;\n", index_VR, con_VR.at(10*index_VR+5));

		}
		if (con_VR.at(10*index_VR+6) != -1) {
		
			fprintf(fp, "C_%d >= %E;\n", index_VR, con_VR.at(10*index_VR+6));

		}
		if (con_VR.at(10*index_VR+7) != -1) {
		
			fprintf(fp, "C_%d <= %E;\n", index_VR, con_VR.at(10*index_VR+7));

		}
		if (con_VR.at(10*index_VR+8) != -1) {
		
			fprintf(fp, "(Ir_%d/8/f_%d/C_%d)*(0.25*Vin_%d/D_%d/(Vin_%d-Vout_%d))*(1/K_%d/K_%d/K_%d) <= %E*Vout_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, con_VR.at(10*index_VR+8), index_VR);

		}
		if (con_VR.at(10*index_VR+9) != -1) {
		
			fprintf(fp, "delta_%d*delta_%d*0.5*Iout_%d*0.5*Iout_%d*(L_%d/C_%d) <= %E*Vout_%d*%E*Vout_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, con_VR.at(10*index_VR+9), index_VR, con_VR.at(10*index_VR+9), index_VR);

		}

		fprintf(fp, "K_%d >= 1;\n", index_VR);

		// decribe the power and area relationship according to different types of the VRs

		if (allocate_VR.at(index_VR) == 1) {
			
			float V_driver = param_type_1.at(6);

			fprintf(fp, "%E*%E/W_%d <= Rds_%d;\n", param_type_1.at(0), param_type_1.at(12)*temp_VR.at(index_VR) + param_type_1.at(13), index_VR, index_VR);
			fprintf(fp, "Ir_%d == (Vin_%d - Vout_%d)*D_%d/f_%d/L_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
			fprintf(fp, "Rind_%d == L_%d/%E*%E;\n", index_VR, index_VR, param_type_1.at(4), 1.0 + material_TC.at(param_type_1.at(16))*(temp_VR.at(index_VR) - 25));
			fprintf(fp, "Iout_%d/Vin_%d*(Rds_%d + Rind_%d + Vout_%d/Iout_%d) <= D_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
			fprintf(fp, "%E*W_%d*%E*%E*f_%d + Rds_%d*(Iout_%d*Iout_%d + 1/12*Ir_%d*Ir_%d) + %E*(Iout_%d + 0.5*Ir_%d)*f_%d*Vin_%d + (Rind_%d)*(Iout_%d*Iout_%d + 1/12*Ir_%d*Ir_%d) + Vout_%d*Iout_%d + %E*%E*W_%d + %E*Vin_%d*K_%d + %E*%E*Iout_%d*Iout_%d<= Vin_%d*Iin_%d;\n", param_type_1.at(1), index_VR, V_driver, V_driver, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, param_type_1.at(2), index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, param_type_1.at(3), param_type_1.at(14)*temp_VR.at(index_VR) + param_type_1.at(15), index_VR, param_type_1.at(7), index_VR, index_VR, param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR, index_VR, index_VR, index_VR);

			fprintf(fp, "delta_%d == %E;\n", index_VR, param_type_1.at(5));
 
			fprintf(fp, "%E*2*W_%d + %E*L_%d*K_%d*K_%d + %E*C_%d + %E*K_%d <= A_%d;\n", param_type_1.at(8), index_VR, param_type_1.at(9), index_VR, index_VR, index_VR, param_type_1.at(10), index_VR, param_type_1.at(11), index_VR, index_VR);
		
		}
		else if (allocate_VR.at(index_VR) == 2) {
		
			fprintf(fp, "%E*%E/W_%d <= Rds_%d;\n", param_type_2.at(0), param_type_2.at(11)*temp_VR.at(index_VR) + param_type_2.at(12), index_VR, index_VR);
			fprintf(fp, "Ir_%d == (Vin_%d - Vout_%d)*D_%d/f_%d/L_%d;\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
			fprintf(fp, "Rind_%d == L_%d/%E*%E;\n", index_VR, index_VR, param_type_2.at(4), 1.0 + material_TC.at(param_type_2.at(15))*(temp_VR.at(index_VR) - 25));
			fprintf(fp, "Iout_%d/Vin_%d*(Rds_%d + Rind_%d + %E*%E*3/8+ Vout_%d/Iout_%d) <= D_%d;\n", index_VR, index_VR, index_VR, index_VR, param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR, index_VR, index_VR);
			fprintf(fp, "%E*W_%d*Vin_%d*Vin_%d*f_%d + Rds_%d*(Iout_%d*Iout_%d + 1/12*Ir_%d*Ir_%d) + %E*(Iout_%d + 0.5*Ir_%d)*f_%d*Vin_%d + (Rind_%d + %E*%E*3/8)*(Iout_%d*Iout_%d + 1/12*Ir_%d*Ir_%d) + Vout_%d*Iout_%d + %E*%E*W_%d + %E*Vin_%d*K_%d + %E*%E*Iout_%d*Iout_%d<= Vin_%d*Iin_%d;\n", param_type_2.at(1), index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, param_type_2.at(2), index_VR, index_VR, index_VR, index_VR, index_VR, param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, param_type_2.at(3), param_type_2.at(13)*temp_VR.at(index_VR) + param_type_2.at(14), index_VR, param_type_2.at(6), index_VR, index_VR, param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR, index_VR, index_VR, index_VR);

			fprintf(fp, "delta_%d == %E;\n", index_VR, param_type_2.at(5));

			fprintf(fp, "%E*W_%d*3 + %E*L_%d*K_%d*K_%d + %E*C_%d + %E*K_%d <= A_%d;\n", param_type_2.at(7), index_VR, param_type_2.at(8), index_VR, index_VR, index_VR, param_type_2.at(9), index_VR, param_type_2.at(10), index_VR, index_VR);

		}
		else {
		
			assert(1 !=1 );
		
		}

		index_VR ++;

	}
}


// print the current relationship

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	if (i == 0) {

		for (int j=0; j<num_VR.at(i); j++) {

			index_VR ++;

		}

	}
	else if ((i>0) && (i<num_stage)) {
	
		for (int j=0; j<num_VR.at(i); j++) {
		

			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
			
				fprintf(fp, "Iin_%d + ", alloc_VR.at(i).at(j).at(k));
			
			}

			fprintf(fp, "0 <= Iout_%d;\n", index_VR);

			index_VR ++;
			
		}
	}
}

// print the area constraint

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	if (i == 0) {

		for (int j=0; j<num_VR.at(i); j++) {

			fprintf(fp, "A_%d + ", index_VR);

			index_VR ++;

		}

		fprintf(fp, "0.0 <= %E;\n", con_area.at(i));
	}
	else if ((i>0) && (i<num_stage)) {
	
		for (int j=0; j<num_VR.at(i); j++) {

			fprintf(fp, "A_%d + ", index_VR);

			index_VR ++;
			
		}

		fprintf(fp, "0.0 <= %E;\n", con_area.at(i));
	
	}
}


fprintf(fp, "cvx_end\n");


fprintf(fp, "if (cvx_optval <= minimum)\n");
fprintf(fp, "minimum = cvx_optval;\n");
fprintf(fp, "fid=fopen('PDS_optvals.txt','w');\n");
fprintf(fp, "fprintf(fid, '%%E\\n\\n', power_proc/Vs/minimum);\n");

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		fprintf(fp, "fprintf(fid, '%%E\\n', Vin_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', Iin_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', Vout_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', Iout_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', Vout_%d*Iout_%d/(Vin_%d*Iin_%d - %E*%E*Iout_%d*Iout_%d));\n", index_VR, index_VR, index_VR, index_VR, param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR, index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', (Ir_%d/8/f_%d/C_%d)*((floor(K_%d+0.5)*D_%d - floor(floor(K_%d+0.5)*D_%d))*(floor(floor(K_%d+0.5)*D_%d) + 1 - floor(K_%d+0.5)*D_%d))*(Vin_%d/D_%d/(Vin_%d-Vout_%d))*(1/floor(K_%d+0.5)/floor(K_%d+0.5)/floor(K_%d+0.5)));\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', delta_%d*sqrt(0.5*Iout_%d*0.5*Iout_%d*(L_%d/C_%d)));\n", index_VR, index_VR, index_VR, index_VR, index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', 2.5*sqrt(2*L_%d*C_%d/((Vout_%d/Vin_%d + 0.6*Vout_%d/Vin_%d)/2*(1-(Vout_%d/Vin_%d + 0.6*Vout_%d/Vin_%d)/2)*(1/abs(Vout_%d/Vin_%d - 0.6*Vout_%d/Vin_%d) + 0.5))));\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
		
		if (allocate_VR.at(index_VR) == 1) {
			fprintf(fp, "fprintf(fid, '%%E\\n', %E*2*W_%d + %E*L_%d*K_%d*K_%d + %E*C_%d + %E*K_%d);\n", param_type_1.at(8), index_VR, param_type_1.at(9), index_VR, index_VR, index_VR, param_type_1.at(10), index_VR, param_type_1.at(11), index_VR);
		}
		else if (allocate_VR.at(index_VR) == 2) {
			fprintf(fp, "fprintf(fid, '%%E\\n', %E*W_%d*3 + %E*L_%d*K_%d*K_%d + %E*C_%d + %E*K_%d);\n", param_type_2.at(7), index_VR, param_type_2.at(8), index_VR, index_VR, index_VR, param_type_2.at(9), index_VR, param_type_2.at(10), index_VR);
		}
		else {
			assert(1 !=1 );
		}
		fprintf(fp, "fprintf(fid, '%%E\\n', W_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', f_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', L_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n', C_%d);\n", index_VR);
		fprintf(fp, "fprintf(fid, '%%E\\n\\n', floor(K_%d+0.5));\n", index_VR);

		index_VR ++;	
	
	}

}

fprintf(fp, "fclose(fid);\n");
fprintf(fp, "end\n");

for (int i=0; i<(inter_level.size()/3); i++) {
			
	fprintf(fp, "end\n");

}


fclose(fp);

// generate the python file for the convex solver of CVXOPT

fp=fopen("./PDS_optimization.py", "w");

if (fp == NULL) {
	cout << "Error opening file" << endl;
	return 0;
}

fprintf(fp, "from cvxopt import matrix, log, exp, solvers\n");

fprintf(fp, "minimum = 1E6;\n");
fprintf(fp, "power_proc = %E;\n", power_proc);
fprintf(fp, "Vs = %E;\n", Vs);

index_VR = 0;

// print the for statement for evaluating different combinations of regulator allocation
for (int i=0; i<num_stage; i++) {

	if (i == 0) {

		for (int j=0; j<num_VR.at(i); j++) {
			
			fprintf(fp, "Vout_%d = max(", index_VR);

			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
			
				fprintf(fp, "%E, ", voltage.at(alloc_VR.at(i).at(j).at(k)));

			}

			fprintf(fp, "0.0)\n");

			fprintf(fp, "Iout_%d = ", index_VR);


			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
			
				fprintf(fp, "%E + ", current.at(alloc_VR.at(i).at(j).at(k)));
			
			}

			fprintf(fp, "0 \n");

			index_VR ++;

		}
		
		for (int i=0; i<(inter_level.size()/3); i++) {

			fprintf(fp, "inter_level_%d = [", i);

			for (int j=0; j<(inter_level.at(3*i)-1); j++) {
			
				fprintf(fp, "%E, ", inter_level.at(3*i+1)+j*(inter_level.at(3*i+2)-inter_level.at(3*i+1))/(inter_level.at(3*i)-1));
			
			}

			fprintf(fp, "%E]\n", inter_level.at(3*i+2));
		}
		

		for (int i=0; i<(inter_level.size()/3); i++) {
			
			for (int j=0; j<i; j++) {
			
				fprintf(fp, "\t");
			
			}
			fprintf(fp, "for inter_%d in inter_level_%d: \n", i, i);
		
		}

		if (num_stage > 1) {

			for (int i=0; i<(inter_level.size()/3); i++) {
		
				for (int j=0; j<(inter_level.size()/3); j++) {
			
					fprintf(fp, "\t");
			
				}

				fprintf(fp, "Vout_%d = inter_%d\n", index_VR+i, i);
		
			}
		}
		else {
		
			for (int i=0; i<(inter_level.size()/3); i++) {
		
				for (int j=0; j<(inter_level.size()/3); j++) {
			
					fprintf(fp, "\t");
			
				}

				fprintf(fp, "Vin_%d = inter_%d\n", i, i);
		
			}
		
		}

	}
	else if ((i>0) && (i<num_stage)) {
	
		for (int j=0; j<num_VR.at(i); j++) {
		
			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {
				
				for (int m=0; m<(inter_level.size()/3); m++) {
			
					fprintf(fp, "\t");
			
				}

				fprintf(fp, "Vin_%d = Vout_%d \n", alloc_VR.at(i).at(j).at(k), index_VR);
			
			}
			if (i == num_stage -1) {
				
				for (int m=0; m<(inter_level.size()/3); m++) {
			
					fprintf(fp, "\t");
			
				}

				fprintf(fp, "Vin_%d = %E \n", index_VR, Vs);
			
			}

			index_VR ++;
		
		}	
	}
}

// derive the sparse matrix 
// ininitalize the matrix used for the CVXOPT

vector < vector < float > > matrix;
vector < int > statement;

vector < float > temp_vector;

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "coef = log( matrix( [");

int num_variable = 0;

temp_val1 = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		if (i == 0) {
		
			num_variable = num_variable + num_variable_1;

		}
		else {
		
			num_variable = num_variable + num_variable_2;

		}
	
	}

}

temp_vector.clear();
temp_vector.resize(num_variable, 0.0);

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		if (i == num_stage -1) {
			
			if (num_stage == 1) {
			
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 10 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "1.0, ");

			}
			else {

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "1.0, ");

			}
		}
		index_VR ++;
	}	
}

statement.push_back(num_VR.at(num_stage-1));

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		// print the design constraint of the VRs
		if (index_VR < num_VR.at(0)) {
		
			temp_vector.clear();
			temp_vector.resize(num_variable, 0.0);

			temp_vector.at(index_VR*num_variable_1 + 7 -1) = 1.0;
			
			matrix.push_back(temp_vector);

			fprintf(fp, "1.0, ");

			statement.push_back(1);

		}
		else {
		
			temp_vector.clear();
			temp_vector.resize(num_variable, 0.0);

			temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = 1.0;
			
			matrix.push_back(temp_vector);

			fprintf(fp, "1.0, ");
			
			statement.push_back(1);
		
		}

		if (con_VR.at(10*index_VR+0) != -1) {
		
			
			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+0));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+0));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+1) != -1) {
		
			
			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+1));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+1));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+2) != -1) {

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 3 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+2));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = -1.0;
			
				matrix.push_back(temp_vector);
			
				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+2));

				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+3) != -1) {
		

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+3));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+3));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+4) != -1) {
		

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 6 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+4));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+4));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+5) != -1) {

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+5));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+5));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+6) != -1) {

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 8 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+6));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 8 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", con_VR.at(10*index_VR+6));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+7) != -1) {
		

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 8 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+7));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 8 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", 1/con_VR.at(10*index_VR+7));
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+8) != -1) {

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 8 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 9 -1) = -3.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(0.125*0.25*Vin_%d/(Vin_%d-Vout_%d)/%E/Vout_%d), ", index_VR, index_VR, index_VR, con_VR.at(10*index_VR+8), index_VR);

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 8 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = -3.0;
			
				matrix.push_back(temp_vector);
		
				fprintf(fp, "abs(0.125*0.25*Vin_%d/(Vin_%d-Vout_%d)/%E/Vout_%d), ", index_VR, index_VR, index_VR, con_VR.at(10*index_VR+8), index_VR);
			
				statement.push_back(1);
		
			}
		}
		if (con_VR.at(10*index_VR+9) != -1) {

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 12 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 8 -1) = -1.0;
			
				matrix.push_back(temp_vector);


				fprintf(fp, "abs(0.5*0.5*Iout_%d*Iout_%d/Vout_%d/Vout_%d/%E/%E), ", index_VR, index_VR, index_VR, index_VR, con_VR.at(10*index_VR+9), con_VR.at(10*index_VR+9));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 13 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 8 -1) = -1.0;
			
				matrix.push_back(temp_vector);
		
				fprintf(fp, "abs(0.5*0.5/Vout_%d/Vout_%d/%E/%E), ", index_VR, index_VR, con_VR.at(10*index_VR+9), con_VR.at(10*index_VR+9));
			
				statement.push_back(1);
		
			}
		}

		if (index_VR < num_VR.at(0)) {
		
			temp_vector.clear();
			temp_vector.resize(num_variable, 0.0);

			temp_vector.at(index_VR*num_variable_1 + 9 -1) = -1.0;
			
			
			matrix.push_back(temp_vector);


			fprintf(fp, "1.0, ");

			statement.push_back(1);

		}
		else {
		
			temp_vector.clear();
			temp_vector.resize(num_variable, 0.0);

			temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = -1.0;
			
			matrix.push_back(temp_vector);
		
			fprintf(fp, "1.0, ");
			
			statement.push_back(1);
		
		}

		// decribe the power and area relationship according to different types of the VRs

		if (allocate_VR.at(index_VR) == 1) {
			
			float V_driver = param_type_1.at(6);
			
			
			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 5 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E), ", param_type_1.at(0), param_type_1.at(12)*temp_VR.at(index_VR) + param_type_1.at(13));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E), ", param_type_1.at(0), param_type_1.at(12)*temp_VR.at(index_VR) + param_type_1.at(13));
			
				statement.push_back(1);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/(Vin_%d-Vout_%d)), ", index_VR, index_VR);

				statement.push_back(1);


				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vin_%d-Vout_%d), ", index_VR, index_VR);

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/(Vin_%d-Vout_%d)), ", index_VR, index_VR);
			
				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vin_%d-Vout_%d), ", index_VR, index_VR);
			
				statement.push_back(1);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E/%E), ", param_type_1.at(4), 1.0 + material_TC.at(param_type_1.at(16))*(temp_VR.at(index_VR) - 25));

				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/%E*%E), ", param_type_1.at(4), 1.0 + material_TC.at(param_type_1.at(16))*(temp_VR.at(index_VR) - 25));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = -1.0;
				
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E/%E), ", param_type_1.at(4), 1.0 + material_TC.at(param_type_1.at(16))*(temp_VR.at(index_VR) - 25));
			
				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/%E*%E), ", param_type_1.at(4), 1.0 + material_TC.at(param_type_1.at(16))*(temp_VR.at(index_VR) - 25));
			
				statement.push_back(1);
		
			}
			
			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 5 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d/Vin_%d), ", index_VR, index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d/Vin_%d), ", index_VR, index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d/Vin_%d), ", index_VR, index_VR);

				statement.push_back(3);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);
			
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d/Vin_%d), ", index_VR, index_VR);

				statement.push_back(3);
		
			}

			if (index_VR < num_VR.at(0)) {
			
				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E*%E/Vin_%d), ", param_type_1.at(1), V_driver, V_driver, index_VR);

				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 5 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d*Iout_%d/Vin_%d), ", index_VR, index_VR, index_VR);

				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 5 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*Iout_%d), ", param_type_1.at(2), index_VR);

				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(0.5*%E), ", param_type_1.at(2));

				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d*Iout_%d/Vin_%d), ", index_VR, index_VR, index_VR);

				//
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d*Iout_%d/Vin_%d), ", index_VR, index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E/Vin_%d), ", param_type_1.at(3), param_type_1.at(14)*temp_VR.at(index_VR) + param_type_1.at(15), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 9 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(7));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E*Iout_%d*Iout_%d/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR, index_VR, index_VR);

				statement.push_back(11);

			}
			else {
		
				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E*%E/Vin_%d), ", param_type_1.at(1), V_driver, V_driver, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(2));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(0.5*%E), ", param_type_1.at(2));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d/Vin_%d), ", index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E/Vin_%d), ", param_type_1.at(3), param_type_1.at(14)*temp_VR.at(index_VR) + param_type_1.at(15), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(7));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR);

				statement.push_back(11);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 12 -1) = 1.0;
				
				matrix.push_back(temp_vector);
		
				fprintf(fp, "abs(1.0/%E), ", param_type_1.at(5));

				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 12 -1) = -1.0;

				matrix.push_back(temp_vector);		

				fprintf(fp, "abs(%E), ", param_type_1.at(5));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 13 -1) = 1.0;
				
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/%E), ", param_type_1.at(5));
			
				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 13 -1) = -1.0;
				
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(5));
			
				statement.push_back(1);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(2.0*%E), ", param_type_1.at(8));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 9 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(9));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 8 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(10));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 9 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(11));

				statement.push_back(4);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(2.0*%E), ", param_type_1.at(8));
			
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(9));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 8 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(10));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_1.at(11));

				statement.push_back(4);
		
			}
		}
		else if (allocate_VR.at(index_VR) == 2) {

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 5 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E), ", param_type_2.at(0), param_type_2.at(11)*temp_VR.at(index_VR) + param_type_2.at(12));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E), ", param_type_2.at(0), param_type_2.at(11)*temp_VR.at(index_VR) + param_type_2.at(12));
			
				statement.push_back(1);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/(Vin_%d-Vout_%d)), ", index_VR, index_VR);

				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vin_%d-Vout_%d), ", index_VR, index_VR);

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/(Vin_%d-Vout_%d)), ", index_VR, index_VR);
			
				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vin_%d-Vout_%d), ", index_VR, index_VR);
			
				statement.push_back(1);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E/%E), ", param_type_2.at(4), 1.0 + material_TC.at(param_type_2.at(15))*(temp_VR.at(index_VR) - 25));

				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = -1.0;
				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/%E*%E), ", param_type_2.at(4), 1.0 + material_TC.at(param_type_2.at(15))*(temp_VR.at(index_VR) - 25));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = -1.0;
							
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E/%E), ", param_type_2.at(4), 1.0 + material_TC.at(param_type_2.at(15))*(temp_VR.at(index_VR) - 25));
			
				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = -1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/%E*%E), ", param_type_2.at(4), 1.0 + material_TC.at(param_type_2.at(15))*(temp_VR.at(index_VR) - 25));
			
				statement.push_back(1);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 5 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d/Vin_%d), ", index_VR, index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d/Vin_%d), ", index_VR, index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0/8*%E*%E*Iout_%d/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR, index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 7 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d/Vin_%d), ", index_VR, index_VR);

				statement.push_back(4);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);
				
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0/8*%E*%E/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR);
				
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 7 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d/Vin_%d), ", index_VR, index_VR);

				statement.push_back(4);
		
			}

			if (index_VR < num_VR.at(0)) {
			
				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*Vin_%d), ", param_type_2.at(1), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 5 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d*Iout_%d/Vin_%d), ", index_VR, index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 5 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*Iout_%d), ", param_type_2.at(2), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 3 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(0.5*%E), ", param_type_2.at(2));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Iout_%d*Iout_%d/Vin_%d), ", index_VR, index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 4 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0/8*%E*%E*Iout_%d*Iout_%d/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR, index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 2 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0/8*%E*%E*1/12/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d*Iout_%d/Vin_%d), ", index_VR, index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E/Vin_%d), ", param_type_2.at(3), param_type_2.at(13)*temp_VR.at(index_VR) + param_type_2.at(14), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 9 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(6));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E*Iout_%d*Iout_%d/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR, index_VR, index_VR);

				statement.push_back(13);

			}
			else {
		
				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*Vin_%d), ", param_type_2.at(1), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 5 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(2));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 3 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(0.5*%E), ", param_type_2.at(2));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 4 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/12/Vin_%d), ", index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0/8*%E*%E/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 2 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0/8*%E*%E*1/12/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_VR.at(index_VR) - 25), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(Vout_%d/Vin_%d), ", index_VR, index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E/Vin_%d), ", param_type_2.at(3), param_type_2.at(13)*temp_VR.at(index_VR) + param_type_2.at(14), index_VR);

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(6));

				// 
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 10 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E*%E/Vin_%d), ", param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR);

				statement.push_back(13);
		
			}

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 12 -1) = 1.0;
				
				matrix.push_back(temp_vector);
		
				fprintf(fp, "abs(1.0/%E), ", param_type_2.at(5));

				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 12 -1) = -1.0;

				matrix.push_back(temp_vector);
		

				fprintf(fp, "abs(%E), ", param_type_2.at(5));

				statement.push_back(1);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 13 -1) = 1.0;
				
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(1.0/%E), ", param_type_2.at(5));
			
				statement.push_back(1);

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 13 -1) = -1.0;
				
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(5));
			
				statement.push_back(1);
		
			}

			// print the area relationship 

			if (index_VR < num_VR.at(0)) {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 1 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0*%E), ", param_type_2.at(7));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 6 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 9 -1) = 2.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(8));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 8 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(9));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(index_VR*num_variable_1 + 9 -1) = 1.0;
				temp_vector.at(index_VR*num_variable_1 + 11 -1) = -1.0;

				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(10));

				statement.push_back(4);

			}
			else {
		
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 1 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(3.0*%E), ", param_type_2.at(7));
			
				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 6 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = 2.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(8));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 8 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(9));

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 9 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = -1.0;
			
				matrix.push_back(temp_vector);

				fprintf(fp, "abs(%E), ", param_type_2.at(10));

				statement.push_back(4);
		
			}
		}
		else {
		
			assert(1 !=1 );
		
		}

		index_VR ++;

	}
}

// print the current relationship

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
		
		if (i == 0) {
		}
		else if (i == 1) {
		
			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(alloc_VR.at(i).at(j).at(k)*num_variable_1 + 10 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = -1.0;
			
				matrix.push_back(temp_vector);
		
				fprintf(fp, "1.0, ");

			}
			
			statement.push_back(alloc_VR.at(i).at(j).size());
		
		}
		else {
		
			for (int k=0; k<alloc_VR.at(i).at(j).size(); k++) {

				temp_vector.clear();
				temp_vector.resize(num_variable, 0.0);

				temp_vector.at(num_VR.at(0)*num_variable_1 + (alloc_VR.at(i).at(j).at(k)-num_VR.at(0))*num_variable_2 + 10 -1) = 1.0;
				temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 11 -1) = -1.0;
			
				matrix.push_back(temp_vector);
		
				fprintf(fp, "1.0, ");
			
			}
			
			statement.push_back(alloc_VR.at(i).at(j).size());

		}
		index_VR ++;
	}	
}

// print the area constraint

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	if (i == 0) {
	
		for (int j=0; j<num_VR.at(i); j++) {
		
			temp_vector.clear();
			temp_vector.resize(num_variable, 0.0);

			temp_vector.at(index_VR*num_variable_1 + 11 -1) = 1.0;
			
			matrix.push_back(temp_vector);
		
			fprintf(fp, "abs(1/%E), ", con_area.at(i));

			index_VR ++;
		
		}

		statement.push_back(num_VR.at(i));
	
	}
	else {
	
		for (int j=0; j<num_VR.at(i); j++) {
		
			temp_vector.clear();
			temp_vector.resize(num_variable, 0.0);

			temp_vector.at(num_VR.at(0)*num_variable_1 + (index_VR-num_VR.at(0))*num_variable_2 + 12 -1) = 1.0;
			
			matrix.push_back(temp_vector);
		
			fprintf(fp, "abs(1/%E), ", con_area.at(i));

			index_VR ++;
		
		}

		statement.push_back(num_VR.at(i));
	
	}
}

temp_vector.clear();
temp_vector.resize(num_variable, 0.0);
matrix.push_back(temp_vector);

fprintf(fp, "0.5]) )\n");

// print the statement

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "state = [");

for (int i=0; i<statement.size(); i++) {

	fprintf(fp, "%d, ", statement.at(i));

}

fprintf(fp, "1]\n");

// print the matrix

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "F = matrix( [");

for (int i=0; i<num_variable; i++) {

	if (i == 0) {
	
		fprintf(fp, "[");

		for (int j=0; j<matrix.size()-1; j++) {
		
			fprintf(fp, "%f, ", matrix.at(j).at(i));
		
		}

		fprintf(fp, "0.0]");
	
	}
	else {
	
		fprintf(fp, ",\n");

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\t\t\t[");

		for (int j=0; j<matrix.size()-1; j++) {
		
			fprintf(fp, "%f, ", matrix.at(j).at(i));
		
		}

		fprintf(fp, "0.0]");

	}

}

fprintf (fp, "])\n");

// print the solver

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
		
		if (i == 0) {
		
			if ((num_stage == 1) && (j == num_VR.at(0)-1)) {
			
				fprintf(fp, "W_%d, Ir_%d, f_%d, Rind_%d, Rds_%d, L_%d, D_%d, C_%d, K_%d, Iin_%d, A_%d, delta_%d = exp ( solvers.gp(state, F, coef)['x'])\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);

			}
			else {
			
				fprintf(fp, "W_%d, Ir_%d, f_%d, Rind_%d, Rds_%d, L_%d, D_%d, C_%d, K_%d, Iin_%d, A_%d, delta_%d, ", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);

			}
		
		}
		else if ((i == num_stage -1) && (j == num_VR.at(i)-1)) {
		
			fprintf(fp, "W_%d, Ir_%d, f_%d, Rind_%d, Rds_%d, L_%d, D_%d, C_%d, K_%d, Iin_%d, Iout_%d, A_%d, delta_%d = exp ( solvers.gp(state, F, coef)['x'])\n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
		}
		else {
		
			fprintf(fp, "W_%d, Ir_%d, f_%d, Rind_%d, Rds_%d, L_%d, D_%d, C_%d, K_%d, Iin_%d, Iout_%d, A_%d, delta_%d, ", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);
		}

		index_VR ++;

	}
}


// calculate the minimum input current

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf (fp, "cvx_optval = ");

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {
	
		if (i == num_stage -1) {

			fprintf(fp, "Iin_%d + ", index_VR);

		}

		index_VR ++;
	}	

}

fprintf(fp, "0.0\n");

// record the optimum results among different intermediate voltage levels

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "if cvx_optval <= minimum: \n");

for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "\tminimum = cvx_optval \n");


for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "\tfid = open('./PDS_optvals.txt', 'w') \n");


for (int m=0; m<(inter_level.size()/3); m++) {
			
	fprintf(fp, "\t");
			
}

fprintf(fp, "\tprint >> fid, \"%%E\\n\" %%(power_proc/Vs/minimum) \n");

index_VR = 0;

for (int i=0; i<num_stage; i++) {

	for (int j=0; j<num_VR.at(i); j++) {

		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(Vin_%d) \n", index_VR);

		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(Iin_%d) \n", index_VR);

		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(Vout_%d) \n", index_VR);

		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(Iout_%d) \n", index_VR);

		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(Vout_%d*Iout_%d/(Vin_%d*Iin_%d - %E*%E*Iout_%d*Iout_%d)) \n", index_VR, index_VR, index_VR, index_VR, param_PDN.at(index_VR), 1.0 + material_TC.at(material_PDN.at(index_VR))*(temp_PDN.at(index_VR) - 25), index_VR, index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%((Ir_%d/8/f_%d/C_%d)*((int(K_%d+0.5)*D_%d - int(int(K_%d+0.5)*D_%d))*(int(int(K_%d+0.5)*D_%d) + 1 - int(K_%d+0.5)*D_%d))*(Vin_%d/D_%d/(Vin_%d-Vout_%d))*(1/int(K_%d+0.5)/int(K_%d+0.5)/int(K_%d+0.5))) \n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(delta_%d*(0.5*Iout_%d*0.5*Iout_%d*(L_%d/C_%d))**0.5) \n", index_VR, index_VR, index_VR, index_VR, index_VR);

		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(2.5*(2*L_%d*C_%d/((Vout_%d/Vin_%d + 0.6*Vout_%d/Vin_%d)/2*(1-(Vout_%d/Vin_%d + 0.6*Vout_%d/Vin_%d)/2)*(1/abs(Vout_%d/Vin_%d - 0.6*Vout_%d/Vin_%d) + 0.5)))**0.5) \n", index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR, index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		if (allocate_VR.at(index_VR) == 1) {
			fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(%E*2*W_%d + %E*L_%d*K_%d*K_%d + %E*C_%d + %E*K_%d)\n", param_type_1.at(8), index_VR, param_type_1.at(9), index_VR, index_VR, index_VR, param_type_1.at(10), index_VR, param_type_1.at(11), index_VR);
		}
		else if (allocate_VR.at(index_VR) == 2) {
			fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(%E*W_%d*3 + %E*L_%d*K_%d*K_%d + %E*C_%d + %E*K_%d)\n", param_type_2.at(7), index_VR, param_type_2.at(8), index_VR, index_VR, index_VR, param_type_2.at(9), index_VR, param_type_2.at(10), index_VR);
		}
		else {
			assert(1 !=1 );
		}
		
		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(W_%d) \n", index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(f_%d) \n", index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(L_%d) \n", index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\t\" %%(C_%d) \n", index_VR);

		for (int m=0; m<(inter_level.size()/3); m++) {
			
			fprintf(fp, "\t");
			
		}

		fprintf(fp, "\tprint >> fid, \"%%E\\n\" %%(int(K_%d)) \n", index_VR);

		index_VR ++;	
	
	}
}

fprintf(fp, "fid.close() \n");

}